@@ -60,6 +60,12 @@ namespace eval ::logcanvas {
       variable revnum_current
       set sel_rev(A) {}
       set sel_rev(B) {}
+      variable search_lastpattern ""
+      variable search_elements [list]
+      variable search_index 0
+      variable search_lastfill ""
+      variable search_lastcase 0
+      variable search_nocase
       variable logcanvas ".logcanvas$my_idx"
 
       gen_log:log T "ENTER [namespace current]"
@@ -172,10 +178,10 @@ namespace eval ::logcanvas {
             $logcanvas.log configure \
                 -command [namespace code {
                     set rev [$logcanvas.up.revA_rvers cget -text] 
-                    if {$rev =="" || [file isdirectory $filename]} {
-                      svn_log $filename
+                    if {$rev == ""} {
+                      svn_log_rev $filename
                     } else {
-                      svn_log $revpath($rev) $filename
+                      svn_log_rev $revpath($rev)
                     }
                  }]
             if {$kind == "directory"} {
@@ -281,7 +287,13 @@ namespace eval ::logcanvas {
              # We have a checked-out local file
              $logcanvas.log configure \
                   -command [namespace code {
-                    cvs_log $filename
+                    set rev [$logcanvas.up.revA_rvers cget -text] 
+                    if {$rev == ""} {
+                      cvs_log_rev "" $filename
+                    } else {
+                      regsub {\.\d+$} $rev {} baserev
+                      cvs_log_rev $baserev $filename
+                    }
                   }]
              $logcanvas.view configure \
                -command [namespace code {
@@ -678,13 +690,15 @@ namespace eval ::logcanvas {
         variable match
         variable fromtags
         variable totags
-        variable xy
+        variable xyw
         variable boxwidth
         variable fromprefix
         variable toprefix
+        variable mrev
         upvar branch branch
 
-        gen_log:log T "ENTER ($x $y $box_width $height $revision)"
+        #gen_log:log T "ENTER ($x $y $box_width $height $revision)"
+        set xyw($revision) [list $x [expr {$y - ($box_height / 4)}] $box_width ]
         # Draw the list of tags
         set tx [expr {$x - $curr(tspcb)}]
         set ty $y
@@ -692,22 +706,16 @@ namespace eval ::logcanvas {
         foreach tag $tlist($revision) {
           gen_log:log D "$revision: tag $tag"
           if {[string match "${fromprefix}_*" $tag]} {
+            set mrev($tag) $revision
             lappend fromtags $tag
             regsub {.*_(.*$)} $tag {\1} tagend
             gen_log:log D "  $tag is a FROM TAG"
             gen_log:log D "  will need a TO TAG ${toprefix}_${revbtag}_$tagend"
             set match($tag) ${toprefix}_${revbtag}_$tagend
-            set boxwidth($tag) $box_width
-            set xy($tag) [list $x [expr {$y - ($box_height / 4)}]]
           }
           if {[string match "${toprefix}_*" $tag]} {
+            set mrev($tag) $revision
             lappend totags $tag
-            regsub {.*_(.*$)} $tag {\1} tagend
-            gen_log:log D "  $tag is a TO TAG"
-            gen_log:log D "  will need a FROM TAG ${toprefix}_${revbtag}_$tagend"
-            set match($tag) ${toprefix}_${revbtag}_$tagend
-            set boxwidth($tag) $box_width
-            set xy($tag) [list $x [expr {$y - ($box_height / 4)}]]
           }
           if {$opt(show_tags)} {
             set my_font $font_norm
@@ -769,7 +777,7 @@ namespace eval ::logcanvas {
         variable revbranches
 
         #gen_log:log T "ENTER ($x $y $root_rev $branch)"
-        gen_log:log D "Drawing branch \"$branch\" rooted at \"$root_rev\""
+        gen_log:log D "Drawing branch \"$branch\" rooted at \"$root_rev\" at ($x $y)"
         # What revisions to show on this branch?
         if {![info exists branchrevs($branch)]} {set branchrevs($branch) {}}
         if {$branchrevs($branch) == {}} {
@@ -838,9 +846,9 @@ namespace eval ::logcanvas {
             set bbox [$logcanvas.canvas bbox ol_x]
           $logcanvas.canvas dtag ol_x
           if {$bbox == {}} {
-          break
-        }
-        gen_log:log D "horizontal overlap with $bbox"
+            break
+          }
+          gen_log:log D "horizontal overlap with $bbox"
           # Move branch to rightmost point of overlapped objects plus some space
           # N.B. +1 because exactly equal counts as an overlap
           set x [expr {[lindex $bbox 2] + $curr(spcx) + 1}]
@@ -1028,7 +1036,7 @@ namespace eval ::logcanvas {
         variable totags {}
         variable toprefix
         variable fromprefix
-        variable xy
+        variable xyw
         variable boxwidth
         variable view_xoff
         variable view_yoff
@@ -1051,7 +1059,9 @@ namespace eval ::logcanvas {
         variable revpath
         variable revkind
         variable revbranches
+        variable revmergefrom
         variable branchrevs
+        variable mrev
         variable match
 
         gen_log:log T "ENTER ($now)"
@@ -1092,6 +1102,10 @@ namespace eval ::logcanvas {
         foreach a [array names $scope\::revbranches] {
           set revbranches($a) [set $scope\::revbranches($a)]
         }
+        catch { unset revmergefrom }
+        foreach a [array names $scope\::revmergefrom] {
+          set revmergefrom($a) [set $scope\::revmergefrom($a)]
+        }
         catch { unset revkind }
         foreach a [array names $scope\::revkind] {
           set revkind($a) [set $scope\::revkind($a)]
@@ -1237,44 +1251,68 @@ namespace eval ::logcanvas {
           gen_log:log D "fromtags: $fromtags"
           gen_log:log D "totags: $totags"
           if {$opt(show_merges)} {
+            # Draw merge arrows derived from tags
             foreach from $fromtags {
-              gen_log:log D " $from"
-              set xfrom [lindex $xy($from) 0]
-              set yfrom [lindex $xy($from) 1]
+              gen_log:log D "  $from on $mrev($from)"
               if {! [info exists match($from)]} {
                 gen_log:log D "  No match for $match($from)"
                 continue
               }
-              gen_log:log D "  need a matching tag $match($from)"
               foreach to $totags {
-                 gen_log:log D "    comparing $match($from) to $to"
                  if {[string equal $to $match($from)]} {
-                    gen_log:log D "  to $to at $xy($to)"
-                    set xto [lindex $xy($to) 0]
-                    set yto [lindex $xy($to) 1]
-                    set xmid $xto
-                    set ymid $yto
-                    if {$xto > $xfrom} {
-                      set xfrom [expr {$xfrom + $boxwidth($from)}]
-                      set yfrom [expr {$yfrom - ($box_height / 2)}]
-                      set yto [expr {$yto - ($box_height / 2)}]
-                      set xmid [expr {$xfrom + (($xto - $xfrom) / 2)}]
-                      set ymid [expr {$yto - $box_height}]
-                    } elseif {$xfrom > $xto} {
-                      set xto [expr {$xto + $boxwidth($to)}]
-                      set xmid [expr {$xto + (($xfrom - $xto) / 2)}]
-                      set ymid [expr {$yto + ($box_height / 2)}]
+                    gen_log:log D "  $to on $mrev($to)"
+                    if {! [info exists revmergefrom($mrev($from))]} {
+                      set revmergefrom($mrev($from)) $mrev($to)
+                      gen_log:log D "Set revmergefrom($mrev($from)) = $mrev($to)"
+                    } else {
+                      gen_log:log D "revmergefrom($mrev($from)) exists. Not changing"
                     }
-                    if {$xto == $xfrom} {
-                      set xmid [expr {$xto - ($boxwidth($from) / 2)}]
-                      set ymid [expr {$yfrom - (($yfrom - $yto) / 2)}]
-                    }
-                    $logcanvas.canvas create line \
-                      $xfrom $yfrom $xmid $ymid $xto $yto \
-                      -arrow first -smooth 1
                  }
               }
             }
+            # Draw merge arrows derived from cvsnt mergepoint or svn 1.5 mergeinfo
+            foreach to [array names revmergefrom] {
+              gen_log:log D "revmergefrom($to) $revmergefrom($to)"
+              set from $revmergefrom($to)
+              gen_log:log D " from $from to $to"
+              if [info exists xyw($from)] {
+                gen_log:log D " xyw($from) $xyw($from)"
+              } else {
+                gen_log:log D " xyw($from) doesn't exist"
+                continue
+              }
+              if [info exists xyw($to)] {
+                gen_log:log D " xyw($to) $xyw($to)"
+              } else {
+                gen_log:log D " xyw($to) doesn't exist"
+                continue
+              }
+              set xto [lindex $xyw($from) 0]
+              set yto [lindex $xyw($from) 1]
+              set bwto [lindex $xyw($from) 2]
+              set xfrom [lindex $xyw($to) 0]
+              set yfrom [lindex $xyw($to) 1]
+              set bwfrom [lindex $xyw($to) 2]
+              set xmid $xto
+              set ymid $yto
+              if {$xto > $xfrom} {
+                set xfrom [expr {$xfrom + $bwfrom}]
+                set yfrom [expr {$yfrom - ($box_height / 2)}]
+                set yto [expr {$yto - ($box_height / 2)}]
+                set xmid [expr {$xfrom + (($xto - $xfrom) / 2)}]
+                set ymid [expr {$yto - $box_height}]
+              } elseif {$xfrom > $xto} {
+                set xto [expr {$xto + $bwto}]
+                set xmid [expr {$xto + (($xfrom - $xto) / 2)}]
+                set ymid [expr {$yto + ($box_height / 2)}]
+              } elseif {$xto == $xfrom} {
+                set xmid [expr {$xto - ($bwfrom / 2)}]
+                set ymid [expr {$yfrom - (($yfrom - $yto) / 2)}]
+              }
+              $logcanvas.canvas create line \
+                  $xfrom $yfrom $xmid $ymid $xto $yto \
+                  -arrow first -smooth 1
+            }
           }
           # Reselect the previously selected revisions
           variable sel_tag
@@ -1302,6 +1340,123 @@ namespace eval ::logcanvas {
         save_options
       }
 
+      # Search functionality for log viewer that searches for strings in the
+      # log windows. It will create a new button and an entry box below the logs. You
+      # can enter a glob-style search pattern in the entry field and click the search
+      # button. With every click (or pressing enter), the log viewer jumps from one
+      # occurrence of the pattern to the next, highlighting it in red.
+      # 
+      # The following special characters are used in the search pattern:
+      # 
+      # *      Matches any sequence of characters in string, including a null string.
+      # 
+      # ?      Matches any single character in string.
+      # 
+      # [chars] Matches any character in the set given by chars. If a sequence of the
+      # form x-y appears in chars, then any character between x and y, inclusive, will
+      # match.
+      # 
+      # \x      Matches the single character x. This provides a way of avoiding the
+      # special interpretation of the characters *?[]\ in pattern.
+      # 
+      # If you only enter "FOO" (without the ") in the entry box, it searches the exact
+      # string "FOO". If you want to search all strings starting with "FOO", you have
+      # to put "FOO*". For all strings containing "FOO", you must put "*FOO*".
+      proc Search {} {
+        variable logcanvas
+        variable font_bold
+        variable search_elements
+        variable search_index
+        variable search_lastfill
+        variable search_lastpattern
+        variable search_lastcase
+        variable search_nocase
+
+        gen_log:log T "ENTER"
+        # Restore last fill color
+        if {[string length $search_lastfill] != 0} {
+          $logcanvas.canvas itemconfigure [lindex $search_elements $search_index] \
+          -fill $search_lastfill
+        }
+        # Read search pattern from entry box
+        set pattern [string trim [$logcanvas.down.search.e get]]
+        gen_log:log D "pattern: $pattern"
+        # Check if search pattern or nocase flag have been changed since the
+        # last call
+        if {([string equal $pattern $search_lastpattern] == 0) \
+          ||($search_lastcase != $search_nocase)} {
+          # Rebuild matching element list
+          set search_lastpattern $pattern
+          set search_lastcase $search_nocase
+          set search_elements [list]
+          # Ignore empty patterns
+          if {[string length $pattern] != 0} {
+            # Loop over all elements in canvas that have text elements
+            foreach element [$logcanvas.canvas find withtag all] {
+              if {[catch {$logcanvas.canvas itemcget $element -text} text] == 0} {
+                # Check if text element matches search pattern
+gen_log:log D " $text"
+                if {[string is true $search_nocase]} {
+                  if {[string match -nocase $pattern $text]} {
+                    # Add element to list of matching elements
+                    lappend search_elements $element
+                  }
+                } else {
+                  if {[string match $pattern $text]} {
+                    # Add element to list of matching elements
+                    lappend search_elements $element
+gen_log:log D " $pattern MATCHED $text"
+                  }
+                }
+              }
+            }
+          }
+          # Reset highlight index
+          set search_index 0
+          # Pattern has not been changed since last call and there have been
+          # matching elements found in the last call
+        } elseif {[llength $search_elements] != 0} {
+          # Select next matching element (restart if last one has been passed)
+          incr search_index
+          if {$search_index >= [llength $search_elements]} {
+            set search_index 0
+            puts -nonewline "\a"
+            flush stdout
+          }
+        }
+        # Check if there are matching elements
+        set length [llength $search_elements]
+        if {$length > 0} {
+          $logcanvas.down.search.l configure -text "[expr {$search_index + 1}] / $length"
+          # Scroll to next matching element
+          set element [lindex $search_elements $search_index]
+          set scrollregion [$logcanvas.canvas cget -scrollregion]
+          set coords [$logcanvas.canvas bbox $element]
+          set sx1 [lindex $scrollregion 0]
+          set sy1 [lindex $scrollregion 1]
+          set sx2 [lindex $scrollregion 2]
+          set sy2 [lindex $scrollregion 3]
+          set ix1 [lindex $coords 0]
+          set iy1 [lindex $coords 1]
+          set ix2 [lindex $coords 2]
+          set iy2 [lindex $coords 3]
+          set xview [$logcanvas.canvas xview]
+          set yview [$logcanvas.canvas yview]
+          set vx1 [lindex $xview 0]
+          set vx2 [lindex $xview 1]
+          set vy1 [lindex $yview 0]
+          set vy2 [lindex $yview 1]
+          set x [expr {(double($ix1 - $sx1) / double($sx2 - $sx1)) -(($vx2 - $vx1) / 2)}]
+          set y [expr {(double($iy1 - $sy1) / double($sy2 - $sy1)) -(($vy2 - $vy1) / 2)}]
+          $logcanvas.canvas xview moveto $x
+          $logcanvas.canvas yview moveto $y
+          set search_lastfill [$logcanvas.canvas itemcget $element -fill]
+          $logcanvas.canvas itemconfigure $element -fill red
+        } else {
+          $logcanvas.down.search.l configure -text "Not found"
+        }
+      }
+
       # Collect the user options from the global set
       set opt(update_drawing) $logcfg(update_drawing)
       set opt(scale) $logcfg(scale)
@@ -1472,6 +1627,7 @@ namespace eval ::logcanvas {
            -command [namespace code\
            "$logcanvas.up.log${fm}_rlogfm.rcomment yview"]
       }
+
       grid columnconf $logcanvas.up 5 -weight 1
       grid $logcanvas.up.lfname -column 0 -row 0 -sticky nw
       grid $logcanvas.up.rfname -column 1 -row 0 -columnspan 5 -sticky ew
@@ -1503,6 +1659,20 @@ namespace eval ::logcanvas {
       # the window is resized smaller
       frame $logcanvas.down -relief groove -border 2
       pack $logcanvas.down -side bottom -fill x
+
+      frame $logcanvas.down.search -relief sunk -bd 2
+      button $logcanvas.down.search.b -text "Find a Revision" -command [namespace code {Search}]
+      entry $logcanvas.down.search.e
+      bind $logcanvas.down.search.e <Return> [namespace code {Search}]
+      label $logcanvas.down.search.l -anchor e -width 10 -text ""
+      checkbutton $logcanvas.down.search.c -anchor e -text "Ignore case" \
+        -variable [namespace current]::search_nocase
+      pack $logcanvas.down.search -side top -fill x
+      pack $logcanvas.down.search.b -side left
+      pack $logcanvas.down.search.e -side left
+      pack $logcanvas.down.search.c -side left
+      pack $logcanvas.down.search.l -side left
+
       # The canvas for the big picture
       canvas $logcanvas.canvas -relief sunken -border 2 \
         -height 300 \
@@ -1638,6 +1808,8 @@ namespace eval ::logcanvas {
         [namespace code "Unselect B"]
 
       focus $logcanvas.canvas
+      # FIXME: Why isn't there a bbox when we get here?
+      # Then the yview moveto doesn't work, although it does in tkinter
       $logcanvas.canvas xview moveto 0
       $logcanvas.canvas yview moveto 0
 
