@@ -377,7 +377,7 @@ proc cvs_unedit {args} {
   foreach file [join $args] {
     # Unedit may hang asking for confirmation if file is not up-to-date
     regsub -all {\$} $file {\$} file
-    set commandline "cvs -n update \"$file\""
+    set commandline "$cvs -n update \"$file\""
     gen_log:log C "$commandline"
     catch {eval "exec $commandline"} view_this
     # Its OK if its locally added
@@ -837,7 +837,7 @@ proc cvs_tag {tagname force b_or_t update args} {
   }
 
   if {$tagname == ""} {
-    cvsfail "You must enter a tag name!" .workdir
+    cvsfail "Please enter a tag name!" .workdir
     return 1
   }
 
@@ -1123,7 +1123,7 @@ proc cvs_status {args} {
   set filelist [join $args]
   # if recurse option is true or there are no selected files, recurse
   set flags ""
-  if {$cvscfg(recurse)} {
+  if {! $cvscfg(recurse)} {
     set flags "-l"
   }
   # if there are selected files, use verbose output
@@ -1566,22 +1566,30 @@ proc cvs_rtag { cvsroot mcode b_or_t force oldtag newtag } {
   global cvscfg
   
   gen_log:log T "ENTER ($cvsroot $mcode $b_or_t $force $oldtag $newtag)"
-  if {$newtag == ""} {
-    cvsfail "You must enter a tag name!" .modbrowse
-    return 1
-  }
 
   set command "$cvs -d \"$cvsroot\" rtag"
-  if {$b_or_t == "branch"} {
-    append command " -b"
-  } 
-  if {$force == "yes"} {
-    append command " -F" 
-  }   
-  if {$oldtag != ""} {
-    append command " -r \"$oldtag\""
+  if {$force == "remove"} {
+    if {$oldtag == ""} {
+      cvsfail "Please enter an Old tag name!" .modbrowse
+      return 1
+    }
+    append command " -d \"$oldtag\" \"$mcode\""
+  } else {
+    if {$newtag == ""} {
+      cvsfail "Please enter a New tag name!" .modbrowse
+      return 1
+    }
+    if {$b_or_t == "branch"} {
+      append command " -b"
+    }
+    if {$force == "yes"} {
+      append command " -F"
+    }
+    if {$oldtag != ""} {
+      append command " -r \"$oldtag\""
+    }
+    append command " \"$newtag\" \"$mcode\""
   }
-  append command " \"$newtag\" \"$mcode\""
 
   set v [::viewer::new "CVS Rtag"]
   $v\::do "$command"
@@ -1620,7 +1628,7 @@ proc cvs_commit_dialog {} {
   }
 
   toplevel .commit
-  grab set .commit
+  #grab set .commit
 
   frame .commit.top -border 8
   frame .commit.vers
@@ -1639,14 +1647,16 @@ proc cvs_commit_dialog {} {
 
   frame .commit.comment
   pack .commit.comment -side top -fill both -expand 1
-  label .commit.lcomment
-  text .commit.tcomment -relief sunken -width 70 -height 10 \
+  label .commit.comment.lcomment -text "Your log message" -anchor w
+  button .commit.comment.history -text "Log History" \
+    -command history_browser
+  text .commit.comment.tcomment -relief sunken -width 70 -height 10 \
     -bg $cvsglb(textbg) -exportselection 1 \
     -wrap word -border 2 -setgrid yes
 
 
   # Explain what it means to "commit" files
-  message .commit.message -justify left -aspect 500 -relief groove \
+  message .commit.message -justify left -aspect 500 -relief groove -bd 2 \
     -text "This will commit changes from your \
            local, working directory into the repository, recursively.
 
@@ -1666,44 +1676,45 @@ proc cvs_commit_dialog {} {
 
   pack .commit.message -in .commit.top -padx 2 -pady 5
 
-
   button .commit.ok -text "OK" -highlightbackground $cvsglb(bg) \
     -command {
-      grab release .commit
+      #grab release .commit
       wm withdraw .commit
-      set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
+      set cvsglb(commit_comment) [string trimright [.commit.comment.tcomment get 1.0 end]]
       cvs_commit $version $cvsglb(commit_comment) $cvsglb(commit_list)
+      commit_history $cvsglb(commit_comment)
     }
   button .commit.apply -text "Apply" -highlightbackground $cvsglb(bg) \
     -command {
-      set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
+      set cvsglb(commit_comment) [string trimright [.commit.comment.tcomment get 1.0 end]]
       cvs_commit $version $cvsglb(commit_comment) $cvsglb(commit_list)
+      commit_history $cvsglb(commit_comment)
     }
   button .commit.clear -text "ClearAll" -highlightbackground $cvsglb(bg) \
     -command {
-      set version ""
-      .commit.tcomment delete 1.0 end
+      set version ""e
+      .commit.comment.tcomment delete 1.0 end
     }
   button .commit.quit -highlightbackground $cvsglb(bg) \
     -command {
-      grab release .commit
+      #grab release .commit
       wm withdraw .commit
     }
  
-  .commit.lcomment configure -text "Your log message" \
-    -anchor w
   .commit.ok configure -text "OK"
   .commit.quit configure -text "Close"
-  pack .commit.lcomment -in .commit.comment \
-    -side left -fill x -pady 3
-  pack .commit.tcomment -in .commit.comment \
-    -side left -fill both -expand 1 -pady 3
+
+  grid columnconf .commit.comment 1 -weight 1
+  grid rowconf .commit.comment 1 -weight 1
+  grid .commit.comment.lcomment -column 0 -row 0
+  grid .commit.comment.tcomment -column 1 -row 0 -rowspan 2
+  grid .commit.comment.history  -column 0 -row 1
 
   pack .commit.ok .commit.apply .commit.clear .commit.quit -in .commit.down \
     -side left -ipadx 2 -ipady 2 -padx 4 -pady 4 -fill both -expand 1
 
   # Fill in the most recent commit message
-  .commit.tcomment insert end $cvsglb(commit_comment)
+  .commit.comment.tcomment insert end [string trimright $cvsglb(commit_comment)]
 
   wm title .commit "Commit Changes"
   wm minsize .commit 1 1
@@ -1911,8 +1922,8 @@ proc cvs_lock {do files} {
     return
   }
   switch -- $do {
-    lock { set commandline "cvs admin -l $files"}
-    unlock { set commandline "cvs admin -u $files"}
+    lock { set commandline "$cvs admin -l $files"}
+    unlock { set commandline "$cvs admin -u $files"}
   }
   set lock_cmd [::exec::new "$commandline"]
   auto_setup_dir $lock_cmd
@@ -2054,7 +2065,7 @@ namespace eval ::cvs_branchlog {
           }
         }
         RCS {
-          set command "rlog $filename"
+          set command "rlog \"$filename\""
           set newlc [logcanvas::new $filename "RCS,loc" [namespace current]]
           set ln [lindex $newlc 0]
           set lc [lindex $newlc 1]
@@ -2232,9 +2243,6 @@ namespace eval ::cvs_branchlog {
               } else {
                 if {$logline == "description:"} {
                   set logstate {S}
-                } elseif {$logline == "----------------------------"} {
-                  # Oops, missed something.
-                  set logstate {V}
                 }
               }
             }
@@ -2245,26 +2253,32 @@ namespace eval ::cvs_branchlog {
               }
             }
             {V} {
-              # Look for a revision number line
-              set rnum [lindex [split $logline] 1]
-              set parts [split $rnum {.}]
-              set revkind($rnum) "revision"
-              if {[llength $parts] == 2} {
-                # A trunk revision but not necessarily 1.x because CVS allows
-                # the first part of the revision number to be changed. We have
-                # to assume that people always increase it if they change it
-                # at all.
-                lappend branchrevs(trunk) $rnum
+              if {! [string match "revision *" $logline] } {
+                # Did they put just the right number of dashes in the comment
+                # to fool us?
+                set logstate {L}
               } else {
-                lappend branchrevs([join [lrange $parts 0 end-1] {.}]) $rnum
+                # Look for a revision number line
+                set rnum [lindex [split $logline] 1]
+                set parts [split $rnum {.}]
+                set revkind($rnum) "revision"
+                if {[llength $parts] == 2} {
+                  # A trunk revision but not necessarily 1.x because CVS allows
+                  # the first part of the revision number to be changed. We have
+                  # to assume that people always increase it if they change it
+                  # at all.
+                  lappend branchrevs(trunk) $rnum
+                } else {
+                  lappend branchrevs([join [lrange $parts 0 end-1] {.}]) $rnum
+                }
+                # Branches for this revision may have already been created
+                # during tag parsing
+                foreach "revwho($rnum) revdate($rnum) revtime($rnum)
+                  revlines($rnum) revstate($rnum) revcomment($rnum)" \
+                  {{} {} {} {} {} {}} \
+                  { break }
+                set logstate {D}
               }
-              # Branches for this revision may have already been created
-              # during tag parsing
-              foreach "revwho($rnum) revdate($rnum) revtime($rnum)
-                revlines($rnum) revstate($rnum) revcomment($rnum)" \
-                {{} {} {} {} {} {}} \
-                { break }
-              set logstate {D}
             }
             {D} {
               # Look for a date line.  This also has the name of the author.
@@ -2353,6 +2367,7 @@ namespace eval ::cvs_branchlog {
 
         if {[llength [array names revkind]] < 1} {
           cvsfail "Log empty.  Check error status of cvs log comand"
+          $lc close invoke
           return
         }
 
