@@ -623,13 +623,19 @@ proc rtag_dialog { cvsroot module b_or_t } {
                .modtag.down.cancel invoke
              "
 
+  button .modtag.down.delete -text "Remove" -highlightbackground $cvsglb(bg) \
+    -command "
+               cvs_rtag $cvsroot $module tag remove \$otag \$ntag; \
+               .modtag.down.cancel invoke
+             "
+
   button .modtag.down.cancel -text "Cancel" -highlightbackground $cvsglb(bg) \
     -command {
                grab release .modtag
                destroy .modtag
              }
 
-  pack .modtag.down.tag .modtag.down.cancel -in .modtag.down -side left \
+  pack .modtag.down.tag .modtag.down.delete .modtag.down.cancel -in .modtag.down -side left \
     -ipadx 2 -ipady 2 -padx 4 -pady 4 -fill both -expand 1
 
   bind .modtag.top.nentry <Return> \
@@ -851,17 +857,20 @@ proc cvs_update_options {} {
   radiobutton .update.options.whichrev.keep \
     -text "Keep same branch or trunk" \
     -variable cvsglb(tagmode_selection) -value "Keep" -anchor w \
-    -command {.update.options.whichrev.getrev.lblentry.tname configure -state disabled}
+    -command {.update.options.whichrev.getrev.lblentry.tname configure -state disabled
+              .update.options.whichrev.getrev.lblentry.dirtag configure -state disabled}
   # update to the head revision
   radiobutton .update.options.whichrev.trunk \
     -text "Update local files to be on main trunk (-A)" \
     -variable cvsglb(tagmode_selection) -value "Trunk" -anchor w \
-    -command {.update.options.whichrev.getrev.lblentry.tname configure -state disabled}
+    -command {.update.options.whichrev.getrev.lblentry.tname configure -state disabled
+              .update.options.whichrev.getrev.lblentry.dirtag configure -state disabled}
   # update to different branch/tag or not
   radiobutton .update.options.whichrev.tag \
     -text "Update (-r) local files to be on tag/branch" \
     -variable cvsglb(tagmode_selection) -value "Getrev" -anchor w \
-    -command {.update.options.whichrev.getrev.lblentry.tname configure -state normal}
+    -command {.update.options.whichrev.getrev.lblentry.tname configure -state normal
+              .update.options.whichrev.getrev.lblentry.dirtag configure -state normal}
 
   message .update.options.whichrev.explainkeep -font $cvscfg(listboxfont) \
     -justify left -width 400 \
@@ -886,6 +895,10 @@ you may want to commit any local changes to that branch first."
   label .update.options.whichrev.getrev.lblentry.tlbl -text "Tag Name" -anchor w
   entry .update.options.whichrev.getrev.lblentry.tname -relief sunken \
     -textvariable cvsglb(updatename)
+  button .update.options.whichrev.getrev.lblentry.dirtag -text "Dir Tag" \
+    -command {
+    set cvsglb(updatename) $current_tagname
+    }
   message .update.options.whichrev.getrev.explaintag -font $cvscfg(listboxfont) \
     -justify left -width 400 \
     -text "Advice:  Update local files to main trunk (head) first.
@@ -895,6 +908,7 @@ Note:  The tag will be 'sticky' for the directory and for each file."
   pack .update.options.whichrev.getrev.lblentry -side top -expand 1 -fill x
   pack .update.options.whichrev.getrev.lblentry.tlbl -side left
   pack .update.options.whichrev.getrev.lblentry.tname -side left -fill x -padx 2 -pady 4
+  pack .update.options.whichrev.getrev.lblentry.dirtag -side left -fill x -padx 2 -pady 4
   pack .update.options.whichrev.getrev.explaintag \
     -side top -fill x -pady 1 -ipady 0
 
@@ -958,6 +972,7 @@ Note:  The tag will be 'sticky' for the directory and for each file."
   # State of top radiobuttons (keep same, main, or tag)
   if {$cvsglb(tagmode_selection) != "Getrev"} {
     .update.options.whichrev.getrev.lblentry.tname configure -state disabled
+    .update.options.whichrev.getrev.lblentry.dirtag configure -state disabled
   }
   # state of -l radiobuttons
   if {$cvsglb(update_recurse) != "recurse"} {
@@ -1463,3 +1478,74 @@ proc dialog_merge_notice {sys from frombranch fromtag totag filelist} {
   }
 }
 
+# Keep a log of commit log messages.  We want to do this whether the
+# history has ever been examined by the user or not
+proc commit_history {comment} {
+  global cvsglb
+
+  set comment [string trimright $comment]
+  set c 0
+  foreach ch [array names cvsglb commit_comment,*] {
+    if {$comment eq $cvsglb($ch)} {
+      # We already have this one.  We don't have to
+      # do anything else.
+      gen_log:log D "Comment is a duplicate"
+      return
+    }
+    incr c
+  }
+  # We don't have this one yet
+  set cvsglb(commit_comment,$c) $comment
+  gen_log:log D "New comment $c"
+  if [winfo exists .ci_history] {
+    .ci_history.text insert end "$comment"
+    .ci_history.text insert end "\n"
+    .ci_history.text insert end "================================================================================\n"
+  }
+}
+
+# See the previous log messages
+proc history_browser {} {
+  global cvsglb
+
+  gen_log:log T "ENTER history_browser"
+
+  if {! [winfo exists .ci_history]} {
+    toplevel .ci_history
+    wm protocol .ci_history WM_DELETE_WINDOW { wm withdraw .ci_history }
+    wm title .ci_history "Commit Log History for Session"
+
+    text .ci_history.text -setgrid yes -relief sunken -border 2 \
+      -exportselection 1 -yscroll ".ci_history.scroll set"
+    scrollbar .ci_history.scroll -relief sunken \
+      -command ".ci_history.text yview"
+    frame .ci_history.bottom
+    search_textwidget_init
+    button .ci_history.bottom.srchbtn -text Search \
+      -highlightbackground $cvsglb(bg) \
+      -command "search_textwidget .ci_history.text"
+    entry .ci_history.bottom.entry -width 20 -textvariable cvsglb(searchstr)
+    bind .ci_history.bottom.entry <Return> \
+        "search_textwidget .ci_history.text"
+    button .ci_history.bottom.close -text "Close" \
+       -highlightbackground $cvsglb(bg) \
+       -command { wm withdraw .ci_history }
+
+    pack .ci_history.bottom -side bottom -fill x
+    pack .ci_history.scroll -side right -fill y
+    pack .ci_history.text -fill both -expand 1
+    pack .ci_history.bottom.srchbtn -side left
+    pack .ci_history.bottom.entry -side left
+    pack .ci_history.bottom.close -side right
+
+    # If this is the first time we've built the window, add the history we have so far
+    foreach ch [array names cvsglb commit_comment,*] {
+      .ci_history.text insert end $cvsglb($ch)
+      .ci_history.text insert end "\n"
+      .ci_history.text insert end "================================================================================\n"
+    }
+  }
+
+  wm deiconify .ci_history
+  gen_log:log T "LEAVE"
+}
