@@ -156,7 +156,7 @@ namespace eval ::exec {
           }
           $v_w.text yview end
         }
-        gen_log:log D "STDOUT:  $line"
+        gen_log:log F "STDOUT:  $line"
       }
 
       proc err_handler {} {
@@ -432,14 +432,6 @@ proc patch_colortags {exec line} {
   return [list $tag $line]
 }
 
-proc parse_version {exec line} {
-  if {[string match "Concurrent*" $line]} {
-    set version [lindex [split $line] 4]
-    return [list tagged $version]
-  }
-  return [list {} {}]
-}
-
 proc hilight_rcslog {exec line} {
   set tag default
   if {[string match "=============*" $line]} {
