@@ -140,69 +140,113 @@ proc read_svn_dir {dirname} {
   return 1
 }
 
+proc svn_lock {do files} {
+  global cvscfg
+
+  if {$files == {}} {
+    cvsfail "Please select one or more files!" .workdir
+    return
+  }
+  switch -- $do {
+    lock { set commandline "svn lock $files"}
+    unlock { set commandline "svn unlock $files"}
+  }
+  set cmd [::exec::new "$commandline"]
+
+  if {$cvscfg(auto_status)} {
+    $cmd\::wait
+    setup_dir
+  }
+}
+
 # Get stuff for main workdir browser
 proc svn_workdir_status {} {
   global cmd
   global Filelist
 
   gen_log:log T "ENTER"
-  set cmd(svn_status) [exec::new "svn status -uvN"]
-  set status_lines [split [$cmd(svn_status)\::output] "\n"]
+  set cmd(svn_status) [exec::new "svn status -uvN --xml"]
+  set xmloutput [$cmd(svn_status)\::output]
+  set entrylist [regexp -all -inline {<entry.*?</entry>} $xmloutput]
+
   if [info exists cmd(svn_status)] {
     $cmd(svn_status)\::destroy
     catch {unset cmd(svn_status)}
   }
-  # The first five columns in the output are each one character wide
-  foreach logline $status_lines {
-    if {[string match "Status*" $logline]} {continue}
-    if {[string length $logline] < 1} {continue}
+  # do very simple xml parsing
+  foreach entry $entrylist {
+    set filename ""
     set cauthor ""
-    set crev ""
+    set lockstatus ""
     set wrev ""
-    set status ""
-    
-    set varcols [string range $logline 8 end]
-    if {[llength $varcols] > 1} {
-      #012345678
-      # M           965       938 kfogel       wc/bar.c
-      #       *     965       922 sussman      wc/foo.c
-      #A  +         965       687 joe          wc/qax.c
-      #             965       687 joe          wc/zig.c
-      set wrev [lindex $varcols 0]
-      set crev [lindex $varcols 1]
-      set nb [string first "/emailAddress=" [lrange $varcols 3 end] ]
-      if {$nb == "-1"} {
-        set cauthor [lindex $varcols 2]
-        set filename [lrange $varcols 3 end]
-      } else {
-        set cauthor [lrange $varcols 2 3]
-        set filename [lrange $varcols 4 end]
+    set crev ""
+
+    regexp  {<entry\s+path=\"([^\"]*?)\"\s*>} $entry tmp filename
+    regexp  {<wc\-status.*</wc\-status>} $entry wcstatusent
+    if { [ regexp  {<repos\-status.*</repos\-status>} $entry repstatusent ] } {
+      regexp  {<repos\-status\s+([^>]*)>} $repstatusent tmp repstatusheader
+      regexp  {item=\"(\w+)\"} $repstatusheader tmp repstatus
+      if { [ regexp  {<lock>.*</lock>} $repstatusent replock ] } {
+        set lockstatus "locked"
       }
     } else {
-      #?                                       newfile
-      set filename [lrange $logline 1 end]
+      set repstatus ""
     }
-    set modstat [string range $logline 0 7]
-    set m1 [string index $modstat 0]
+    regexp  {<author>(.*)</author>} $wcstatusent tmp cauthor
+    regexp  {<commit\s+revision=\"(\d+)\"} $wcstatusent tmp crev
+    regexp  {<wc\-status\s+([^>]*)>} $wcstatusent tmp wcstatusheader
+    regexp  {item=\"(\w+)\"} $wcstatusheader tmp wcstatus
+    regexp  {revision=\"(\w+)\"} $wcstatusheader tmp wrev
+    if { [ regexp  {<lock>.*</lock>} $wcstatusent wclock ] } {
+      set lockstatus "havelock"
+    }
+
+    # wcstatus="added|normal|deleted|unversioned|modified|none
+    # repstatus="modified|none"
+    set status ""
+
     set displaymod ""
+    if { [file exists $filename] && [file type $filename] == "link" } {
+        set displaymod "<link> "
+    }
     if [file isdirectory $filename] {
       set displaymod "<dir> "
     }
-    switch -exact -- $m1 {
-      " " { append displaymod "Up-to-date" }
-      M { append displaymod "Locally Modified" }
-      A { append displaymod "Locally Added" }
-      D { append displaymod "Locally Removed" }
-      ? { append displaymod "Not managed by SVN" }
-      C { append displaymod "Conflict" }
+
+    set mayhavelock false
+    switch -exact -- $wcstatus {
+      "normal" {
+        if { $repstatus == "modified"} {
+          append displaymod "Out-of-date"
+        } else {
+          append displaymod "Up-to-date"
+          set mayhavelock true
+        }
+      }
+      "modified" {
+        if  { $repstatus == "modified"} {
+          append displaymod "Needs Merge"
+        } else {
+          append displaymod "Locally Modified"
+           set mayhavelock true
+        }
+      }
+      "added" { append displaymod "Locally Added" }
+      "deleted" { append displaymod "Locally Removed" }
+      "unversioned" { append displaymod "Not managed by SVN" }
+      "conflicted" { append displaymod "Conflict" }
       L { append displaymod "Locked" }
       S { append displaymod "Switched to Branch" }
-      ! { append displaymod "Missing or Incomplete" }
+      "none" { append displaymod "Missing/Needs Update" }
       ~ { append displaymod "Dir/File Mismatch" }
     }
-
-    if {[string index $modstat 7] == "*"} {
-       set displaymod "Out-of-date"
+    #in some cases there might be locks: check now
+    if { $mayhavelock } {
+        switch -exact -- $lockstatus {
+            "" { }
+            "havelock" { append displaymod "/HaveLock" }
+            "locked" { append displaymod "/Locked" }
+        }
     }
     set Filelist($filename:wrev) $wrev
     set Filelist($filename:status) $displaymod
@@ -213,6 +257,12 @@ proc svn_workdir_status {} {
     }
     set Filelist($filename:option) ""
     set Filelist($filename:editors) "$cauthor"
+    #gen_log:log D " \
+       \"$Filelist($filename:status)\" \
+       \"$wrev (committed:$crev)\" \
+       \"$Filelist($filename:editors)\" \
+       \"$filename\" \
+       "
   }
   gen_log:log T "LEAVE"
 }
@@ -229,7 +279,7 @@ proc svn_add {args} {
     set mess "This will add these files:\n\n"
     foreach file $filelist {
       append mess "   $file\n"
-    }  
+    }
   }
 
   if {$filelist == ""} {
@@ -305,7 +355,7 @@ proc svn_update {args} {
   }
   append mess "\n\nAre you sure?"
 
-  set command "svn update"
+  set command "svn update --accept postpone"
 
   if {[cvsconfirm $mess .workdir] == "ok"} {
     foreach file $filelist {
@@ -370,7 +420,7 @@ proc svn_commit_dialog {} {
   }
 
   toplevel .commit
-  grab set .commit
+  #grab set .commit
 
   frame .commit.top -border 8
   frame .commit.down -relief groove -border 2
@@ -379,8 +429,10 @@ proc svn_commit_dialog {} {
   pack .commit.down -side bottom -fill x
   frame .commit.comment
   pack .commit.comment -side top -fill both -expand 1
-  label .commit.lcomment
-  text .commit.tcomment -relief sunken -width 70 -height 10 \
+  label .commit.comment.lcomment -text "Your log message" -anchor w
+  button .commit.comment.history -text "Log History" \
+    -command history_browser
+  text .commit.comment.tcomment -relief sunken -width 70 -height 10 \
     -bg $cvsglb(textbg) -exportselection 1 \
     -wrap word -border 2 -setgrid yes
 
@@ -392,44 +444,45 @@ proc svn_commit_dialog {} {
 
   pack .commit.message -in .commit.top -padx 2 -pady 5
 
-
   button .commit.ok -text "OK" -highlightbackground $cvsglb(bg) \
     -command {
-      grab release .commit
+      #grab release .commit
       wm withdraw .commit
-      set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
+      set cvsglb(commit_comment) [.commit.comment.tcomment get 1.0 end]
       svn_commit $cvsglb(commit_comment) $cvsglb(commit_list)
+      commit_history $cvsglb(commit_comment)
     }
   button .commit.apply -text "Apply" -highlightbackground $cvsglb(bg) \
     -command {
-      set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
+      set cvsglb(commit_comment) [.commit.comment.tcomment get 1.0 end]
       svn_commit $cvsglb(commit_comment) $cvsglb(commit_list)
+      commit_history $cvsglb(commit_comment)
     }
   button .commit.clear -text "ClearAll" -highlightbackground $cvsglb(bg) \
     -command {
       set version ""
-      .commit.tcomment delete 1.0 end
+      .commit.comment.tcomment delete 1.0 end
     }
   button .commit.quit -highlightbackground $cvsglb(bg) \
     -command {
-      grab release .commit
+      #grab release .commit
       wm withdraw .commit
     }
- 
-  .commit.lcomment configure -text "Your log message" \
-    -anchor w
+
   .commit.ok configure -text "OK"
   .commit.quit configure -text "Close"
-  pack .commit.lcomment -in .commit.comment \
-    -side left -fill x -pady 3
-  pack .commit.tcomment -in .commit.comment \
-    -side left -fill both -expand 1 -pady 3
+
+  grid columnconf .commit.comment 1 -weight 1
+  grid rowconf .commit.comment 1 -weight 1
+  grid .commit.comment.lcomment -column 0 -row 0
+  grid .commit.comment.tcomment -column 1 -row 0 -rowspan 2
+  grid .commit.comment.history  -column 0 -row 1
 
   pack .commit.ok .commit.apply .commit.clear .commit.quit -in .commit.down \
     -side left -ipadx 2 -ipady 2 -padx 4 -pady 4 -fill both -expand 1
 
   # Fill in the most recent commit message
-  .commit.tcomment insert end $cvsglb(commit_comment)
+  .commit.comment.tcomment insert end $cvsglb(commit_comment)
 
   wm title .commit "Commit Changes"
   wm minsize .commit 1 1
@@ -538,7 +591,7 @@ proc svn_patch { pathA pathB revA dateA revB dateB outmode outfile } {
 # If both are null the top two revisions of the file are diffed.
 #
   global cvscfg
- 
+
   gen_log:log T "ENTER ($pathA $pathB $revA $dateA $revB $dateB $outmode $outfile)"
   global cvs
 
@@ -636,8 +689,10 @@ proc svn_jit_listdir { tf into } {
     if [string match {*/} $logline] {
       set item [lrange $logline 5 end]
       set item [string trimright $item "/"]
-      lappend dirs "$item"
-      set info($item) [lrange $logline 0 4]
+      if {$item ne "."} {
+        lappend dirs "$item"
+        set info($item) [lrange $logline 0 4]
+      }
     } else {
       set item [lrange $logline 6 end]
       lappend fils "$item"
@@ -744,7 +799,9 @@ proc parse_svnmodules {tf svnroot} {
     gen_log:log D "$logline"
     if [string match {*/} $logline] {
       set item [lrange $logline 5 end]
+        if {$item ne "./"} {
       lappend dirs [string trimright $item "/"]
+      }
     } else {
       set item [lrange $logline 6 end]
       lappend fils $item
@@ -819,6 +876,24 @@ proc svn_log_rev {filepath} {
   gen_log:log T "LEAVE"
 }
 
+proc svn_info {args} {
+  global cvscfg
+  gen_log:log T "ENTER ($args)"
+
+  set filelist [join $args]
+  set urllist ""
+  foreach file $filelist {
+      append urllist $cvscfg(url)/$file
+      append urllist " "
+  }
+  set command "svn info "
+  append command $urllist
+
+  set logcmd [viewer::new "SVN Info ($cvscfg(ldetail))"]
+  $logcmd\::do "$command"
+  gen_log:log T "LEAVE"
+}
+
 proc svn_merge_conflict {args} {
   global cvscfg
 
@@ -844,8 +919,8 @@ proc svn_merge_conflict {args} {
     }
     gen_log:log F "CLOSE $file"
     close $f
-   
-    if { $match != 1 } { 
+
+    if { $match != 1 } {
       cvsfail "$file does not appear to have a conflict." .workdir
       continue
     }
@@ -867,7 +942,7 @@ proc svn_merge_conflict {args} {
       cvsfail "$view_this" .workdir
     }
   }
-  
+
   if {$cvscfg(auto_status)} {
     setup_dir
   }
@@ -934,7 +1009,7 @@ proc svn_tag {tagname b_or_t update args} {
   global cvsglb
 
   gen_log:log T "ENTER ($tagname $b_or_t $update $args)"
-  
+
   if {$tagname == ""} {
     cvsfail "You must enter a tag name!" .workdir
     return 1
@@ -1193,7 +1268,7 @@ proc svn_checkout {dir url path rev target cmd} {
   gen_log:log T "ENTER ($dir $url $path $rev $target $cmd)"
 
   foreach {incvs insvn inrcs} [cvsroot_check $dir] { break }
-  if {$insvn} { 
+  if {$insvn} {
     set mess "This is already a SVN controlled directory.  Are you\
               sure that you want to export into this directory?"
     if {[cvsconfirm $mess .modbrowse] != "ok"} {
@@ -1204,7 +1279,7 @@ proc svn_checkout {dir url path rev target cmd} {
   set command "svn $cmd"
   if {$rev != {} } {
     # Let them get away with saying r3 instead of 3
-    set rev [string trimleft $rev {r}] 
+    set rev [string trimleft $rev {r}]
     append command " -r$rev"
   }
   set path [safe_url $path]
@@ -1286,7 +1361,7 @@ proc svn_fileview {revision filename kind} {
 proc svn_directory_merge {} {
   global cvscfg
   global cvsglb
-  
+
   gen_log:log T "ENTER"
 
   gen_log:log D "Relative Path: $cvsglb(relpath)"
@@ -1299,7 +1374,7 @@ proc svn_directory_merge {} {
 proc svn_branches {files} {
   global cvscfg
   global cvsglb
-  
+
   gen_log:log T "ENTER ($files)"
   set filelist [join $files]
 
@@ -1476,7 +1551,8 @@ namespace eval ::svn_branchlog {
         gen_log:log C "$command"
         set ret [catch {eval "exec $command"} output]
         if {$ret} {
-          cvsfail $output
+          gen_log:log D "This file $path must not be in the trunk"
+          ## cvsfail $output
         }
         foreach infoline [split $output "\n"] {
           if {[string match "Revision*" $infoline]} {
@@ -1520,7 +1596,7 @@ namespace eval ::svn_branchlog {
               set spl [split $line ":"]
               set fromrevs [lindex $spl 1]
               gen_log:log D "  to $r  fromrevs $fromrevs"
-              regsub {^.*-} $fromrevs {} lastfromrev 
+              regsub {^.*-} $fromrevs {} lastfromrev
               set revmergefrom($r) "r$lastfromrev"
               gen_log:log D "  revmergefrom($r) $revmergefrom($r)"
             }
@@ -1547,6 +1623,10 @@ namespace eval ::svn_branchlog {
         set revbtags($rr) "trunk"
         set revpath($rr) $path
 
+        # if root is not empty added it to the branchlist
+        if { $rr ne "" } {
+          lappend branchlist $rr
+        }
         # Branches
         # Get a list of the branches from the repository
         set command "svn list $cvscfg(svnroot)/$cvscfg(svn_branchdir)"
@@ -1599,7 +1679,7 @@ namespace eval ::svn_branchlog {
                 set spl [split $line ":"]
                 set fromrevs [lindex $spl 1]
                 gen_log:log D "  to $r  fromrevs $fromrevs"
-                regsub {^.*-} $fromrevs {} lastfromrev 
+                regsub {^.*-} $fromrevs {} lastfromrev
                 set revmergefrom($r) "r$lastfromrev"
                 gen_log:log D "  revmergefrom($r) $revmergefrom($r)"
               }
@@ -1622,6 +1702,9 @@ namespace eval ::svn_branchlog {
           }
           set branchrevs($rb) $branchrevs($branch)
           set revkind($rb) "branch"
+          # build a list of all branches so we can make sure each branch is on
+          # a revbranch list so there will be a full set of branches on diagram
+          lappend branchlist $rb
           set revname($rb) $branch
           lappend revbtags($rb) $branch
           set revpath($rb) $path
@@ -1714,11 +1797,11 @@ namespace eval ::svn_branchlog {
             set revkind($rb) "tag"
             set revname($rb) "$tag"
             set revpath($rb) $path
-  
+
             # Now do log -q to find the previous rev, which is down
             # the list.  For tags, it's only one down, so we can limit
             # the log to 2.  It only speeds it up a little though.
-            set command "svn log -q -l 2 $path"
+            set command "svn log -q --limit 2 $path"
             set cmd_log [exec::new $command {} 0 {} 1]
             set log_output [$cmd_log\::output]
             $cmd_log\::destroy
@@ -1735,6 +1818,77 @@ namespace eval ::svn_branchlog {
           }
         }
 
+        # sort the list in rev number order
+        set brlist [lsort -dictionary $branchlist]
+        gen_log:log D "init branches $brlist"
+        # rebuild the list
+        set branchlist {}
+        foreach br $brlist {
+          lappend branchlist $br
+          # add to the list any revs that are in the branch revs
+          # that also have revbranches
+          if {[info exists branchrevs($br)]} {
+            foreach r $branchrevs($br) {
+              if {[info exists revbranches($r)] } {
+                lappend branchlist $r
+              }
+            }
+          }
+        }
+        set branchlist [lsort -dictionary $branchlist]
+        gen_log:log D "branches $branchlist"
+
+        # add any branches that are not on a revbranches list to the one closest
+        # in numeric value
+
+        # counter of branches in the list
+        set brn 0
+        # get the length of the list so we can tell when we are done
+        set brlistlen [llength $branchlist]
+        while {$brn<$brlistlen} {
+          # get the branch name
+          set br [lindex $branchlist $brn]
+          gen_log:log D "  branch $brn is $br"
+          # look at all the branches up to branch $br
+          set subbrn 0
+          set subbrwithrevs r0
+          set subbrwithrevsnum 0
+          set foundinrevbr 0
+          while {$subbrn<$brn} {
+            set subbr [lindex $branchlist $subbrn]
+            # check each revbranch for this branch
+            if {[info exists revbranches($subbr)]} {
+              # remember the highest number rev with revbranches
+              set subbrnum [string trimleft $subbr "r"]
+              if { $subbrwithrevsnum < $subbrnum }  {
+                set subbrwithrevs  $subbr
+                set subbrwithrevsnum  $subbrnum
+              }
+              foreach r $revbranches($subbr) {
+                if {$r==$br} {
+                  # we found it in a revbranches
+                  incr foundinrevbr
+                  break
+                }
+              }
+            }
+            if {$foundinrevbr>0} {
+              gen_log:log D "   found $br in revbranch of $subbr"
+              break
+            }
+            incr subbrn
+          }
+          if {$foundinrevbr<=0 && $subbrwithrevsnum!=0} {
+            # we only want to attach a branch & not a rev that a branch is attached
+            if { $revkind($br) eq "branch" } {
+              gen_log:log D "   put $br in revbranches of $subbrwithrevs"
+              lappend revbranches($subbrwithrevs) $br
+            } else {
+              gen_log:log D "   branch $br not attached because not a real branch"
+            }
+          }
+          incr brn
+        }
         pack forget $lc.stop
         pack $lc.close -in $lc.down.closefm -side right
         $lc.close configure -state normal
@@ -1779,7 +1933,7 @@ namespace eval ::svn_branchlog {
             gen_log:log D "revdate($revnum) $revdate($revnum)"
             gen_log:log D "revtime($revnum) $revtime($revnum)"
             gen_log:log D "notelen $notelen"
-            
+
             incr i 2
             set revcomment($revnum) ""
             set c 0
@@ -1833,7 +1987,7 @@ namespace eval ::svn_branchlog {
         variable revnum
         variable rootbranch
         variable revbranch
-  
+
         gen_log:log T "ENTER"
 
         # Sort the revision and branch lists and remove duplicates
@@ -1850,7 +2004,9 @@ namespace eval ::svn_branchlog {
            gen_log:log D "branchrevs($a) $branchrevs($a)"
         }
         gen_log:log D ""
-           foreach a [lsort -dictionary [array names revbranches]] {
+        foreach a [lsort -dictionary [array names revbranches]] {
+           # sort the rev branches to they will be displayed in increasing order
+           set revbranches($a) [lsort -dictionary $revbranches($a)]
            gen_log:log D "revbranches($a) $revbranches($a)"
         }
         gen_log:log D ""
