@@ -1,3 +1,4 @@
+
 #!/bin/sh
 #-*-tcl-*-
 # the next line restarts using wish \
@@ -13,6 +14,10 @@ exec wish "$0" -- ${1+"$@"}
 
 # If we can't get this far (maybe because X display connection refused)
 # quit now.  If we get further, the error message is very misleading.
+if {[info exists starkit::topdir]} {
+  package require Tk
+}
+
 if {! [info exists tk_version] } {
    puts "Initialization failed"
    exit 1
@@ -23,48 +28,53 @@ if {$tk_version < 8.4} {
   exit 1
 }
 
-if {[info exists TclRoot]} {
-   # Perhaps we are being sourced recursively.
-   # That would be bad.
-   return
+if {[info exists starkit::topdir]} {
+  set TclRoot [file join $starkit::topdir lib]
+  set ScriptBin $starkit::topdir
+} else {
+  if {[info exists TclRoot]} {
+     # Perhaps we are being sourced recursively.
+     # That would be bad.
+     return
+  }
+  set Script [info script]
+  set ScriptTail [file tail $Script]
+  #puts "Tail $ScriptTail"
+  if {[file type $Script] == "link"} {
+    #puts "$Script is a link"
+    set ScriptBin [file join [file dirname $Script] [file readlink $Script]]
+  } else {
+    set ScriptBin $Script
+  }
+  #puts "  ScriptBin $ScriptBin"
+  set TclRoot [file join [file dirname $ScriptBin]]
+  #puts "TclRoot $TclRoot"
+  if {$TclRoot == "."} {
+    set TclRoot [pwd]
+  }
+  #puts "TclRoot $TclRoot"
+  set TclRoot [file join [file dirname $TclRoot] "lib"]
+  #puts "TclRoot $TclRoot"
+  
+  # allow runtime replacement
+  if {[info exists env(TCLROOT)]} {
+   set TclRoot $env(TCLROOT)
+  }
+  #puts "TclRoot $TclRoot"
 }
 
-set Script [info script]
-set ScriptTail [file tail $Script]
-#puts "Tail $ScriptTail"
-if {[file type $Script] == "link"} {
-  #puts "$Script is a link"
-  set ScriptBin [file join [file dirname $Script] [file readlink $Script]]
-} else {
-  set ScriptBin $Script
-}
-#puts "  ScriptBin $ScriptBin"
 set TclExe [info nameofexecutable]
 if {$tcl_platform(platform) == "windows"} {
   set TclExe [file attributes $TclExe -shortname]
 }
 
-set TclRoot [file join [file dirname $ScriptBin]]
-#puts "TclRoot $TclRoot"
-if {$TclRoot == "."} {
-  set TclRoot [pwd]
-}
-#puts "TclRoot $TclRoot"
-set TclRoot [file join [file dirname $TclRoot] "lib"]
-#puts "TclRoot $TclRoot"
-
-# allow runtime replacement
-if {[info exists env(TCLROOT)]} {
-   set TclRoot $env(TCLROOT)
-}
-#puts "TclRoot $TclRoot"
 
 set TCDIR [file join $TclRoot tkcvs]
 set cvscfg(bitmapdir) [file join $TclRoot tkcvs bitmaps]
 #puts "TCDIR $TCDIR"
 #puts "BITMAPDIR $cvscfg(bitmapdir)"
 
-set cvscfg(version) "8.2"
+set cvscfg(version) "8.2.1"
 
 if {! [info exists cvscfg(editorargs)]} {
   set cvscfg(editorargs) {}
@@ -122,17 +132,19 @@ if {! [get_cde_params]} {
   destroy .native
   # Hilight colors.  Get the colorful ones.
   entry .testent
-  #set cvsglb(textfg) [lindex [.testent configure -foreground] 4]
   set cvsglb(textbg) white
   set cvsglb(textfg) black
   set cvsglb(readonlybg) gray96
   set cvsglb(hlbg) [lindex [.testent configure -selectbackground] 4]
   set cvsglb(hlfg) [lindex [.testent configure -selectforeground] 4]
+  if {$cvsglb(hlfg) eq {} } {
+    # This happens on the Mac
+    set cvsglb(hlfg) [lindex [.testent configure -foreground] 4]
+  }
   set hlbg_rgb [winfo rgb .testent $cvsglb(hlbg)]
   set cvscfg(listboxfont) [lindex [.testent configure -font] 4]
   destroy .testent
 
-
   # Find out what the default gui font is
   label .testlbl -text "LABEL"
   if { [info exists cvscfg(guifont)] } {
@@ -212,7 +224,7 @@ set cvscfg(flashfont) [list $ffam $fsiz underline]
 #puts "actual flashfont: [font actual $cvscfg(flashfont)]"
 # Prefer underline, but it isn't working at all in tk8.5 on linux
 if {! [font actual $cvscfg(flashfont) -underline]} {
-  puts "Underline font not working.  Trying $ffam $fsiz bold"
+  #puts "Underline font not working.  Trying $ffam $fsiz bold"
   set cvscfg(flashfont) [list [lindex $lbf 1] $fsiz bold]
   #set cvscfg(flashfont) [list [lindex $lbf 1] [lindex $lbf 3] bold]
 }
@@ -259,34 +271,34 @@ for {set i 0} {$i < [llength $argv]} {incr i} {
   set arg [lindex $argv $i]
   set val [lindex $argv [expr {$i+1}]]
   switch -regexp -- $arg {
-    {--*d.*} {
+    {^--*d.*} {
       # -ddir: Starting directory
       set dir $val; incr i
       cd $val
     }
-    {--*r.*} {
+    {^--*r.*} {
       # -root: CVS root
       set cvscfg(cvsroot) $val; incr i
     }
-    {--*w.*} {
+    {^--*w.*} {
       # workdir|module|merge: window to start with. workdir is default.
       set cvscfg(startwindow) $val; incr i
     }
-    {--*l.*} {
+    {^--*l.*} {
       # -log <filename>: Browse the log of specified file
       set cvscfg(startwindow) log
       set lcfile $val; incr i
     }
-    {--*[ab].*} {
+    {^--*[ab].*} {
       # annotate|blame: Browse colorcoded history of specified file
       set cvscfg(startwindow) blame
       set lcfile $val; incr i
     }
-    -psn_* {
+    {^-psn_.*} {
       # Ignore the Carbon Process Serial Number
       incr i
     }
-    {--*h.*} {
+    {^--*h.*} {
       puts $usage
       exit 0
     }
@@ -302,6 +314,13 @@ for {set i 0} {$i < [llength $argv]} {incr i} {
   }
 }
 
+if {[info exists lcfile]} {
+  set d [file dirname $lcfile]
+  set f [file tail $lcfile]
+  set lcfile $f
+  cd $d
+}
+
 # If CVSROOT envvar is set, use it
 if { ! [info exists cvscfg(cvsroot)] } {
   if { ! [info exists env(CVSROOT)] } {
