@@ -150,7 +150,7 @@ proc workdir_setup {} {
      -command { file_input_and_do "New Directory" workdir_newdir}
 
   button .workdir.bottom.buttons.dirfuncs.brefresh -image Refresh -highlightbackground $cvsglb(bg) \
-     -command { change_dir [pwd] }
+     -command { setup_dir }
   button .workdir.bottom.buttons.dirfuncs.bcheckdir -image Check
 
   button .workdir.bottom.buttons.cvsfuncs.blogfile -image Branches -highlightbackground $cvsglb(bg) \
@@ -316,7 +316,8 @@ proc workdir_setup {} {
     wm deiconify .workdir
   }
 
-  change_dir "[pwd]"
+  #change_dir "[pwd]"
+  setup_dir
   gen_log:log T "LEAVE"
 }
 
@@ -526,6 +527,7 @@ proc workdir_menus {} {
   .workdir.menubar.reports add command -label "Status" -underline 0
   .workdir.menubar.reports add command -label "Log" -underline 0
   .workdir.menubar.reports add command -label "Annotate/Blame" -underline 0
+  .workdir.menubar.reports add command -label "Info" -underline 0
 
   .workdir.menubar.options add checkbutton -label "Show hidden files" \
      -variable cvscfg(allfiles) -onvalue true -offvalue false \
@@ -694,7 +696,6 @@ proc workdir_newdir {file} {
   if {$cvscfg(auto_status)} {
     setup_dir
   }
-
   gen_log:log T "LEAVE"
 }
 
@@ -863,6 +864,9 @@ proc change_dir {new_dir} {
     return
   }
   set cwd $new_dir
+  # Deleting the tree discards the saved scroll position
+  # so we start with yview 0 in a new directory
+  DirCanvas:deltree .workdir.main
   setup_dir
 
   gen_log:log T "LEAVE"
@@ -895,12 +899,17 @@ proc setup_dir { } {
 
   gen_log:log T "ENTER"
 
+  set savyview 0
   if { ! [winfo exists .workdir.main] } {
     workdir_setup
     return
   } else {
+    if {[winfo exists .workdir.main.filecol.list]} {
+      set savyview [lindex [.workdir.main.filecol.list yview] 0]
+    }
     DirCanvas:deltree .workdir.main
   }
+  gen_log:log D "YVIEW $savyview"
 
   if {![file isdirectory $cwd]} {
     gen_log:log D "$cwd is not a directory"
@@ -929,6 +938,7 @@ proc setup_dir { } {
   .workdir.menubar.reports entryconfigure 1 -state disabled
   .workdir.menubar.reports entryconfigure 2 -state disabled
   .workdir.menubar.reports entryconfigure 3 -state disabled
+  .workdir.menubar.reports entryconfigure 4 -state disabled
   # Start with the revision-control buttons disabled and the
   .workdir.bottom.buttons.dirfuncs.bcheckdir configure -state disabled
   foreach widget [grid slaves .workdir.bottom.buttons.cvsfuncs ] {
@@ -977,6 +987,7 @@ proc setup_dir { } {
     .workdir.menubar.reports entryconfigure 2 -state normal \
        -command { rcs_log [workdir_list_files] }
     .workdir.menubar.reports entryconfigure 3 -state disabled
+    .workdir.menubar.reports entryconfigure 4 -state disabled
   } elseif {$insvn} {
     # Top
     .workdir.top.bmodbrowse configure -image Modules_svn \
@@ -1011,6 +1022,10 @@ proc setup_dir { } {
       -command { svn_revert [workdir_list_files] }
     .workdir.bottom.buttons.cvsfuncs.btag configure -state normal
     .workdir.bottom.buttons.cvsfuncs.bbranchtag configure -state normal
+    .workdir.bottom.buttons.oddfuncs.block configure -state normal \
+      -command { svn_lock lock [workdir_list_files] }
+    .workdir.bottom.buttons.oddfuncs.bunlock configure -state normal \
+      -command { svn_lock unlock [workdir_list_files] }
     # Menus
     .workdir.menubar entryconfigure "CVS" -state disabled
     .workdir.menubar entryconfigure "SVN" -state normal
@@ -1028,6 +1043,9 @@ proc setup_dir { } {
     # Annotate/Blame (svn blame)
     .workdir.menubar.reports entryconfigure 3 -state normal \
        -command { svn_annotate BASE [workdir_list_files] }
+    # Info (svn info)
+    .workdir.menubar.reports entryconfigure 4 -state normal \
+       -command { svn_info [workdir_list_files] }
   } elseif {$incvs} {
     # Top
     .workdir.top.bmodbrowse configure -image Modules_cvs \
@@ -1092,6 +1110,7 @@ proc setup_dir { } {
     # Annotate/Blame (cvs annotate)
     .workdir.menubar.reports entryconfigure 3 -state normal \
        -command { cvs_annotate $current_tagname [workdir_list_files] }
+    .workdir.menubar.reports entryconfigure 4 -state disabled
   }
 
   DirCanvas:create .workdir.main
@@ -1126,6 +1145,9 @@ proc setup_dir { } {
 
   set filelist [ getFiles ]
   directory_list $filelist
+  # Update, otherwise it won't be mapped before we restore the scroll position
+  update
+  DirCanvas:yview_windows .workdir.main $savyview
 
   gen_log:log T "LEAVE"
 }
